"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/* eslint-disable functional/prefer-immutable-types */
const utils_1 = require("@typescript-eslint/utils");
const common_1 = require("./common");
const fp_ts_1 = require("./fp-ts");
/**
 * An ESLint rule to ban nested fp-ts effects.
 */
// eslint-disable-next-line total-functions/no-unsafe-readonly-mutable-assignment
const noNestedFpTsEffects = (0, common_1.createRule)({
    name: "no-nested-fp-ts-effects",
    meta: {
        type: "problem",
        docs: {
            description: "Bans nested fp-ts effects.",
            recommended: "error",
        },
        messages: {
            errorStringGeneric: "Do not nest effects.",
        },
        schema: [],
    },
    create: (context) => {
        const parserServices = utils_1.ESLintUtils.getParserServices(context);
        const checker = parserServices.program.getTypeChecker();
        return {
            // eslint-disable-next-line functional/no-return-void
            CallExpression: (node) => {
                const tsNode = parserServices.esTreeNodeToTSNodeMap.get(node);
                const expressionType = checker.getTypeAtLocation(tsNode);
                const effectType = (0, fp_ts_1.fpTsEffectType)(expressionType);
                // eslint-disable-next-line functional/no-conditional-statements
                if (effectType === undefined) {
                    return;
                }
                const nestedEffectType = (0, fp_ts_1.fpTsEffectType)(effectType.typeParameter);
                // eslint-disable-next-line functional/no-conditional-statements
                if (nestedEffectType === undefined) {
                    return;
                }
                // eslint-disable-next-line functional/no-expression-statements
                context.report({
                    node: node,
                    messageId: "errorStringGeneric",
                });
            },
        };
    },
    defaultOptions: [],
});
exports.default = noNestedFpTsEffects;
